#ifndef FILE_SYS_H
#define FILE_SYS_H

/*
 * Copyright (C) 2003-4 by Dusan Ferbas - DF Soft
 * <a href="www.dfsoft.cz">www.dfsoft.cz</a>
 *
 * Permission to use, copy, modify, and distribute this software
 * for any purpose and without fee is hereby granted, provided
 * that the above copyright notices appear in all copies and that
 * both the copyright notice and this permission notice appear in
 * supporting documentation and that the name of DF Soft may not be
 * used to endorse or promote products derived from this 
 * software without specific prior written permission.
 *
 * This software is provided "as is" without express or implied
 * warranty. We are not liable for anything caused by using this software.
 *
 * author <a href="mailto:rhlebak@dfsoft.cz">Rostislav Hlebak</a>
 */

/****************************************************************************
 *
 * File:         file_sys.h
 * Author(s):    Rostislav Hlebak
 * Project:      Charon SNMP Implementation
 * Description:  SPI DataFlash file system emulation
 *
 ****************************************************************************/

#include <sys/types.h>


#define FILE_NAME_LENGTH				12	// xxxxxxxx.xxx - 8.3 (12 chars)
#define FILE_COUNT						30	// fits max 30 files (eeprom limit)
#define FILE_SYS_EEPROM_OFFSET			3456	// 0xD80

//******************************************************************************
//          DISPLAY TABLE COMMANDS
//******************************************************************************

#define  FULL_TABLE		1
#define  ACTUALL_TABLE	0
//******************************************************************************
//******************************************************************************
//          WRITE STATE
//******************************************************************************
#define WRITE_BUSY      				1
#define WRITE_READY     				0
//******************************************************************************

//******************************************************************************
//          ERRORS
//******************************************************************************
#define ERR_NOSPACE						-1	// insufficient space in flash
#define ERR_NOFILE						-2	// file does not exist
#define ERR_FILEEXIST					-3	// file already exists
#define ERR_WRITE						-4	// write error
#define ERR_NOTLAST						-5	// file is not last one (delete)
#define ERR_NOMEM						-6	// insufficient memory
#define ERR_READONLY					-7	// file is read only
#define ERR_WRITEONLY					-8	// file is write only
#define ERR_FILESIZE					-9	// undefined file size
//******************************************************************************

//******************************************************************************
//          FLAG OPENAS
//******************************************************************************
#define F_OPENAS_READ					1	// file is read only
#define F_OPENAS_WRITE					2	// file is write only (delete allowed)
#define F_EOF							128	// set if file end reached
//******************************************************************************


//******************************************************************************
//          STRUCTURES
//******************************************************************************
typedef struct
{
	unsigned char file_name[FILE_NAME_LENGTH + 1];	// file name
	unsigned int first_page;				// page where file starts
	unsigned int pages_occupied;			// how many pages it occupies
	unsigned long file_length;				// real file length
}
TFileRecS;

typedef struct
{
	TFileRecS *file_recp;					// file info
	unsigned int file_index;				// file index in table
	unsigned int buffer_counter;			// actual position within a page
	unsigned int page_counter;				// aktualni pozice stranky v pameti
	unsigned char open_as;					// file access - write or read-only
}
Tflash_FILE;

typedef struct
{
	unsigned char file_sysID[4];			// file system ID
	unsigned int pages_used;				// flash pages used
	unsigned int file_count;				// file count
	unsigned int file_state;				// write access restriction - Busy
	TFileRecS file_rec[FILE_COUNT];			// file info
}
TFileTableS;

//******************************************************************************

#ifdef FILESYS_VARS
TFileTableS *gFileTable;
u_char  gFileSysWriteEnable = 0;
#else
extern TFileTableS *gFileTable;
extern u_char gFileSysWriteEnable;

#endif


//******************************************************************************
//          FUNCTION PROTOTYPES
//******************************************************************************

int     flash_InitFileSys(void);			// file system initialization
int     flash_EndFileSys(void);				// file system shutdown
Tflash_FILE *flash_open(u_char * filename, u_char cmd);	// open file
int     flash_close(Tflash_FILE * fd);		// close file
int     flash_read(Tflash_FILE * fd, u_char * buffer, u_long length);	// read buffer from file
int     flash_write(Tflash_FILE * fd, u_char * buffer, u_long length);	// write buffer to file
long    flash_length(Tflash_FILE * fd);		// return file length
int     flash_delete(Tflash_FILE * fd);		// delete last file in case it has write access
int     flash_clear(void);					// erase whole table in RAM and eeprom
long    flash_getfreemem(void);				// free space in flash
int     flash_EOF(Tflash_FILE * fd);		// test for end of file


void    MemStat(void);
void    display_table(u_char cmd);

//******************************************************************************

#endif
