#ifndef SPI_FLASH_H
#define SPI_FLASH_H

/*
 * Copyright (C) 2003-4 by Dusan Ferbas - DF Soft
 * <a href="www.dfsoft.cz">www.dfsoft.cz</a>
 *
 * Permission to use, copy, modify, and distribute this software
 * for any purpose and without fee is hereby granted, provided
 * that the above copyright notices appear in all copies and that
 * both the copyright notice and this permission notice appear in
 * supporting documentation and that the name of DF Soft may not be
 * used to endorse or promote products derived from this 
 * software without specific prior written permission.
 *
 * This software is provided "as is" without express or implied
 * warranty. We are not liable for anything caused by using this software.
 *
 * author <a href="mailto:rhlebak@dfsoft.cz">Rostislav Hlebak</a>
 */

/****************************************************************************
 *
 * File:         spi_flash.h
 * Author(s):    Rostislav Hlebak, Dusan Ferbas
 * Project:      Charon SNMP Implementation
 * Description:  SPI DataFlash low level, based on Atmel AN:
 *				 http://www.atmel.com/dyn/resources/prod_documents/doc1456.pdf
 *
 ****************************************************************************/

//#define SPI_DEBUG


#include <stdio.h>
#include <sys/types.h>


#define SPI_FLASH_M5


// buffer 1
#define BUFFER_1 0x00
// buffer 2
#define BUFFER_2 0x01

//**********************************************************************
// DataFlash pin selection
//**********************************************************************

// DataFlash reset port pin (PB 0)
//#define DF_RESET 0x01

// DataFlash boot sector write protection (PB 2)
//#define DF_WRITE_PROTECT 0x04

// DataFlash ready/busy status port pin PB6 (PB 1)
#define DF_RDY_BUSY 0x40					//0x02

// DataFlash SCK  [PB 2]
#define DF_SCK 0x02

// DataFlash SI  [PB 3]
#define DF_SI 0x04

// DataFlash SO  [PB 4]
#define DF_SO 0x08

// DataFlash chip select port pin PB0 (PB 4)
#define DF_CHIP_SELECT 0x01					//0x10


//**********************************************************************
// defines for all opcodes
//**********************************************************************

// buffer 1 write
#define BUFFER_1_WRITE 0x84

// buffer 2 write
#define BUFFER_2_WRITE 0x87

// buffer 1 read (change to 0xD4 for SPI mode 0,3)
#define BUFFER_1_READ 0x54					//0x54

// buffer 2 read (change to 0xD6 for SPI mode 0,3)
#define BUFFER_2_READ 0x56

// buffer 1 to main memory page program with built-in erase
#define B1_TO_MM_PAGE_PROG_WITH_ERASE 0x83

// buffer 2 to main memory page program with built-in erase
#define B2_TO_MM_PAGE_PROG_WITH_ERASE 0x86

// buffer 1 to main memory page program without built-in erase
#define B1_TO_MM_PAGE_PROG_WITHOUT_ERASE 0x88

// buffer 2 to main memory page program without built-in erase
#define B2_TO_MM_PAGE_PROG_WITHOUT_ERASE 0x89

// main memory page program through buffer 1
#define MM_PAGE_PROG_THROUGH_B1 0x82

// main memory page program through buffer 2
#define MM_PAGE_PROG_THROUGH_B2 0x85

// auto page rewrite through buffer 1
#define AUTO_PAGE_REWRITE_THROUGH_B1 0x58

// auto page rewrite through buffer 2
#define AUTO_PAGE_REWRITE_THROUGH_B2 0x59

// main memory page compare to buffer 1
#define MM_PAGE_TO_B1_COMP 0x60

// main memory page compare to buffer 2
#define MM_PAGE_TO_B2_COMP 0x61

// main memory page to buffer 1 transfer
#define MM_PAGE_TO_B1_XFER 0x53

// main memory page to buffer 2 transfer
#define MM_PAGE_TO_B2_XFER 0x55

// DataFlash status register for reading density, compare status,
// and ready/busy status (change to 0xD7 for SPI mode 0,3)
#define STATUS_REGISTER 0x57

// main memory page read (change to 0xD2 for SPI mode 0,3)
#define MAIN_MEMORY_PAGE_READ 0x52

// erase a 528 byte page
#define PAGE_ERASE 0x81

// erase 512 pages
#define BLOCK_ERASE 0x50

//read froma main memory without using buffer (0xE8 for mode 0, 3)
#define CONTINUOUS_ARRAY_READ 0x68


//#define TRUE 0xff
//#define FALSE 0x00

//**********************************************************************
// other definition
//**********************************************************************

#ifdef SPI_FLASH_M5
#define FLASH_PAGE_SIZE 	264
#define FLASH_PAGE_COUNT 	2048
#define PAGE_ADDRESS		7

#endif

#ifdef SPI_FLASH_2M
#define FLASH_PAGE_SIZE 	528
#define FLASH_PAGE_COUNT 	4096
#define PAGE_ADDRESS		6
#endif






//void page_to_ram(unsigned int page_counter, unsigned char * my_buffer, unsigned int buf_length);
void    SPI_init_flash_port(void);

void    SPI_active_buffer_to_page(unsigned char active_buffer, unsigned int page_counter);
void    SPI_page_to_active_buffer(unsigned char active_buffer, unsigned int page_counter);
//void page_to_active_buffer (unsigned char active_buffer, unsigned int page_counter);

int     SPI_read(unsigned char active_buffer, unsigned char *my_buffer,
				 unsigned int buf_length, unsigned int *buffer_counter,
				 unsigned int *page_counter, unsigned int first_page,
				 unsigned long file_length);


int     SPI_write(unsigned char active_buffer, unsigned char *my_buffer,
				  unsigned int buf_length, unsigned int *buffer_counter,
				  unsigned int *page_counter, unsigned int first_page);

void    active_buffer_to_ram(unsigned char active_buffer, unsigned char *my_buffer,
							 unsigned int buf_length);

//void ram_to_active_buffer (unsigned char active_buffer, unsigned char * my_buffer, unsigned int my_buf_length, unsigned int position);
//void ram_to_page (unsigned char active_buffer, unsigned int page_counter, unsigned char * my_buffer, unsigned int buf_length);
//void page_to_ram (unsigned int page_counter, unsigned int position, unsigned char * my_buffer, unsigned int buf_length);

#endif
