#ifndef EEPROM_H
#define EEPROM_H

/*
 * Copyright (C) 2003-4 by Dusan Ferbas - DF Soft
 * <a href="www.dfsoft.cz">www.dfsoft.cz</a>
 *
 * Permission to use, copy, modify, and distribute this software
 * for any purpose and without fee is hereby granted, provided
 * that the above copyright notices appear in all copies and that
 * both the copyright notice and this permission notice appear in
 * supporting documentation and that the name of DF Soft may not be
 * used to endorse or promote products derived from this
 * software without specific prior written permission.
 *
 * This software is provided "as is" without express or implied
 * warranty. We are not liable for anything caused by using this software.
 *
 * author <a href="mailto:dferbas@dfsoft.cz">Dusan Ferbas</a>
 */

/****************************************************************************
 *
 * File:         eeparams.h
 * Author(s):    Dusan Ferbas
 * Project:      WEB51 Charon SNMP Implementation
 * Description:  eeprom data upgrade/downgrade functionality
 *
 ****************************************************************************/

#include "stddef.h"							//offsetof
#include "msg.h"							/* VER_NUM_STR */

#include <sys/types.h>
#include <snmp/snmp.h>						/* MAX_STRING_LEN */




// ethernet address 6 bytes
typedef struct
{
	u_char  eth_addr[6];
}
ETH_ADDR;

typedef struct
{
	u_char  ip_addr[4];
}
IP_ADDR;


#define EE_ETH_ADDR_OFFSET 0				// MAC address stored at eeprom addres
									  /* (defined only for EEPROM upgrade from nontable version) */
/* --- This section can be customized --- */

#define MAX_SYS_LEN		MAX_STRING_LEN
#define MAX_COMM_LEN	MAX_STRING_LEN

#define USER_SETTINGS_COUNT     1
#define SNMP_TRAPS_COUNT        1
#define SNMP_ACCESS_COUNT       2
#define ALLOWED_IP_RANGE_COUNT	1
#define BAUDRATE_COUNT          10


typedef enum
{ br300, br600, br1200, br2400, br4800, br9600, br19200, br38400, br57600, br115200 }
TBaudRate;
typedef enum
{ db5, db6, db7, db8 }
TDataBits;
typedef enum
{ sb1, sb15, sb2 }
TStopBits;
typedef enum
{ pNone, pEven, pOdd }
TParity;
typedef enum
{ off, on, unknown }
TOffOnUnkn;





typedef struct
{
	IP_ADDR ip;
	IP_ADDR mask;
}
TIpMask;

typedef struct
{
	u_char  Name[32 + 1];
	u_char  Password[32 + 1];
}
TNamePwd;



typedef struct
{
	//------------------------ COMMUNICATION GROUP ---------------------------------
	TBaudRate BaudRate;						//nevyuzito
	TDataBits DataLength;					//nevyuzito
	TStopBits StopBits;						//nevyuzito
	TParity Parity;							//nevyuzito
	u_int   CommTimeout;

	//----------------------- NETWORK SETTINGS GROUP -------------------------------
	IP_ADDR aux_my_ip_addr;					// Web51 local IP address
	IP_ADDR aux_my_ip_mask;					// net mask
	IP_ADDR aux_gw_ip_addr;					// gateway IP address
	u_short http_port;
	//----------------------- USER SETTINGS GROUP ----------------------------------
	TNamePwd UserSettings[USER_SETTINGS_COUNT];	// user levels


}
BoardDataS;

/* default variable values */

/* EEPROM variable types: 0 - 255 (byte value)
 * (do not use previous value for st new !
 * - different customers can have different table versions) */

#define EE_VAR_BAUDRATE             10
#define EE_VAR_DATA_LENGTH          11
#define EE_VAR_STOP_BITS            12
#define EE_VAR_PARITY               13
#define EE_VAR_COMM_TIMEOUT         14

#define	EE_VAR_IPA					 0
#define	EE_VAR_MASK					 1
#define	EE_VAR_GW					 2
#define EE_VAR_HTTP_PORT			 6
#define EE_VAR_USER_SETTINGS		120


#define	EE_VAR_NUM		10					/* needed to fill data structure easily (see eeprom.c)
											 * (count this number easily as substracting source line numbers
											 * in editor - 1st past last line and 1st one)  */
/* --- end of customize section --- */


//redefine to leave old access for old modules
#define my_eth_addr     (g_ee_data.enet_addr)
#define my_ip_addr      (gBoardDatap->aux_my_ip_addr)
#define my_ip_mask      (gBoardDatap->aux_my_ip_mask)
#define gw_ip_addr      (gBoardDatap->aux_gw_ip_addr)

#define CFG_ITEM_ADDR(item)   (u_char *)(&gBoardDatap->item)
#define CFG_ITEM_OFFS(item)   offsetof(BoardDataS, item)
#define CFG_ITEM_LEN(item)    sizeof(gBoardDatap->item)
#define CFG_ITEM_BYTE(item)   (gBoardDatap->item)
#define CFG_ITEM_WORD(item)   (gBoardDatap->item)	//compiler will do the type test
#define CFG_ITEM_LONG(item)   (gBoardDatap->item)	//

// ------------------------------------------------------------------------ //
// -------------      EEPROM data structures                --------------- //
// ------------------------------------------------------------------------ //

#define	EEPROM_TBL_PTRN	"EDTB"				/* this pattern confirms that EEPROM has table structure */
#define eeprom_tbl_tst(ee_tblp) (*(unsigned long *)ee_tblp->t_ptrn == *(unsigned long *)EEPROM_TBL_PTRN)

#define	EE_TABLEI		0x100/*0x60*/		/* data storage place */
/*
 * If around 3500 bytes are enough, check the app/nutpiper/config.c on
 * how to use the internal EEPROM. Note that bytes 0..511 are reserved
 * for the system.
 * 
 * Harald wrote this into conference on 9th December 2003.
 * 
 * (0x60 was working fine with Nut/OS 3.3.2)
 * 
 */
#define	EE_VPTRN		">>> Data for Sensor SNMP Supervisor " VER_NUM_STR " <<<"
								/* this pattern determines table version */
#define	EE_VPTRN_SIZE	(sizeof(EE_VPTRN) - 1)

typedef struct
{
	ETH_ADDR enet_addr;
}
ee_dataS;

typedef struct
{
	u_char  t_ptrn[4];
	u_short t_offs;
	u_short t_count;
	ee_dataS dta;							/* variable(s) that are not reset to default value
											 * during power up reset init should be placed here */
}
ee_tbl;

typedef struct
{
	u_char  v_type;
	u_short v_len;							/* use u_char if you do not have large structures */
	u_short v_offs;							/* use u_char if you do not have too many parameters (they should fit in 255 bytes area) */
}
ee_tbl_elem;

typedef struct
{
	ee_tbl  e_tbl;
	u_char  e_ptrn[EE_VPTRN_SIZE];
	ee_tbl_elem e_elem[EE_VAR_NUM];
}
ee_tbl_data;

typedef struct
{
	u_long  pattern;
	u_long  mask;
}
AccessS;

#ifdef EEPROM_VARS
u_short eeprom_idx = 0;						//this will hold offset to start of gBoardData area in EEPROM
BoardDataS *gBoardDatap;					// v teto strukture budou ulozeny data z eeprom
ee_dataS g_ee_data;							/*this variable will hold fixed data area from EEPROM
											 * (MAC, currently nothing more) */
AccessS gSnmpAccess[SNMP_ACCESS_COUNT];		//computed access masks
AccessS gHttpAccess[ALLOWED_IP_RANGE_COUNT];
u_long  Baudrate_Array[BAUDRATE_COUNT] =
	{ 300, 600, 1200, 2400, 4800, 9600, 19200, 38400, 57600, 115200 };
#else
extern u_short eeprom_idx;
extern BoardDataS *gBoardDatap;
extern ee_dataS g_ee_data;
extern AccessS gSnmpAccess[SNMP_ACCESS_COUNT];
extern AccessS gHttpAccess[ALLOWED_IP_RANGE_COUNT];
extern u_long Baudrate_Array[BAUDRATE_COUNT];
#endif


// ------------------------------------------------------------------------ //

u_char  eeprom_get(u_short addr, u_char * bufp, u_short count);
u_char  eeprom_put(u_short addr, u_char * bufp, u_short count);
u_char  eeprom_load(u_char init_flg);

u_char  eeprom_save(void);
int     eeprom_verify(void);
int     eeprom_clear(void);


#endif
