/*
 * Copyright (C) 2001-2003 by egnite Software GmbH. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY EGNITE SOFTWARE GMBH AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL EGNITE
 * SOFTWARE GMBH OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.ethernut.de/
 *
 */

/*
 * Portions Copyright (C) 2003-4 by Dusan Ferbas - DF Soft
 * <a href="www.dfsoft.cz">www.dfsoft.cz</a>
 *
 * Permission to use, copy, modify, and distribute this software
 * for any purpose and without fee is hereby granted, provided
 * that the above copyright notices appear in all copies and that
 * both the copyright notice and this permission notice appear in
 * supporting documentation and that the name of DF Soft may not be
 * used to endorse or promote products derived from this
 * software without specific prior written permission.
 *
 * This software is provided "as is" without express or implied
 * warranty. We are not liable for anything caused by using this software.
 *
 * author <a href="mailto:dferbas@dfsoft.cz">Dusan Ferbas</a>
 */

/****************************************************************************
 *
 * File:         httpd.c
 * Description:  Modified Nut/OS httpd example, integration with SPI DataFlash
 *				 file system emulation
 *
 ****************************************************************************/


#include <stdio.h>
#include <io.h>
#include <stdlib.h>
#include <string.h>
#include <sys/socket.h>
#include <sys/heap.h>
#include <sys/thread.h>
#include <sys/event.h>
#include <sys/timer.h>

#include <netinet/tcp.h>					//TCP_MAXSEG
#include <arpa/inet.h>						//inet_ntoa() (debug only)

#include "../globdef.h"
#include "../utilities.h"
#include "httpstru.h"
#include "httpinfo.h"						//ShowThreads() - for debug only
#include "../eeparams/eeparams.h"
#include "../spi_flash/file_sys.h"
#include "../spi_flash/spi_flash.h"



#define METHOD_GET		1
#define METHOD_HEAD		2
#define METHOD_POST		3
#define METHOD_PUT		4


#define HTTPD_TCP_MSS	(1500 - 40)			//TCP_MSS/*1024/*(1500 - 40)*/   //override default TCP_MSS == 536
					//IP portion, +14 -> whole packet (1514)

#define HTTP_THREAD_COUNT   3
#define HTTP_PRIO			80
#define HTTP_PRIO_HIGH		30

#define PARSE_LINE_BUF  	256
#define RECEIVE_FILE_BUF	256

////////////////////////////////////////////////////////////////////////////////
void parse_strcpy(u_char * des, u_char * src)
{
	while (*src != ' ')
	{
		if (*src == '%')
		{
			*des++ = hexTochar(++src);
			src++;
			src++;
		}
		else
			*des++ = *src++;
	}
	*des = 0;
}



static void HeaderSuccFileTrf(FILE * stream)
{
	static prog_char header_P[] =
		"HTTP/1.0 200 OK\r\n" "Server: shServer\r\n" "Connection: close\r\n\r\n";
	fprintf_P(stream, header_P);
}
static void SendHeader(FILE * stream, long bytes)
{
	static prog_char header_P[] =
		"HTTP/1.0 200 OK\r\n"
		"Server: shServer\r\n" "Content-Length: %ld\r\n" "Connection: close\r\n\r\n";
	fprintf_P(stream, header_P, bytes);
}

static void SendXMLHeader(FILE * stream)
{
	static prog_char header_P[] =
		"HTTP/1.0 200 OK\r\n"
		"Server: shServer\r\n" "Content-Type: text/html\r\n" "Connection: close\r\n\r\n";

	fprintf_P(stream, header_P);
}

////////////////////////////////////////////////////////////////////////////////
static void HttpSendError(FILE * stream, int stat, HttpRequestS * req)
{
	struct
	{
		int     code;
		char   *text;
	}
	Status[] =
	{
		{201, "Created"},
/*		{304, "Not Modified"},*/
/*		{400, "Bad Request"}, */
		{401, "Unauthorized"},
		{404, "Not Found, Upload your desired files first."},
		{413, "Request Entity Too Large"},
		{500, "Internal Server Error"},
/*		{501, "Not Implemented"}, */
/*		{503, "Service Unavailable"}, */
	};
#define STATUS_CNT  (int)(sizeof(Status)/sizeof(Status[0]))

	int     i;

	for (i = 0; i < STATUS_CNT; i++)
	{
		if (Status[i].code == stat)
			break;
	}

	if (i < STATUS_CNT)						//just to be sure (possible mistype in source code)
	{
		static prog_char auth_fmt_P[] = "WWW-Authenticate: Basic realm=\"%s\"\r\n";
		static prog_char err_fmt_P[] =
			"\r\n"
			"<HTML><HEAD><TITLE>%d %s</TITLE></HEAD><BODY><h1>%d %s</h1></BODY></HTML>";

		fprintf_P(stream, PSTR("HTTP/1.0 %d %s\r\nConnection: close\r\n"), Status[i].code,
				  Status[i].text);

		if (stat == 401)
		{
			char   *cp = NULL;
			char   *realm = req->url;

			if ((cp = strrchr(realm, '/')) != 0)
				*cp = '\0';					//put temporary terminator
			else
				realm = "SPI Flash Demo";
			fprintf_P(stream, auth_fmt_P, realm);
			if (cp)
				*cp = '/';
		}

		/* visible text for user */
		fprintf_P(stream, err_fmt_P, Status[i].code, Status[i].text, Status[i].code,
				  Status[i].text);
		//fflush(stream);
		LOG_MSG("\r\n ERROR: %d %s", Status[i].code, Status[i].text);
	}
}

////////////////////////////////////////////////////////////////////////////////
static void SendWebFile_Flash(FILE * stream, HttpRequestS * http_req)
{
	u_char *data;
	Tflash_FILE *myfile;
	/*
	 * fwrite() calls internally NutTcpDeviceWrite() which will allocate
	 * a buffer of HTTPD_TCP_MSS size.
	 */
	//  printf("\n\rRequest: %s",http_req->url);
	data = (u_char *) NutHeapAlloc(RECEIVE_FILE_BUF);
	if ((myfile = flash_open(http_req->url, F_OPENAS_READ)) != NULL)
	{
		SendHeader(stream, flash_length(myfile));
		int     a;
		do
		{
			a = flash_read(myfile, data, RECEIVE_FILE_BUF);
			fwrite(data, 1, a, stream);
		}
		while (!flash_EOF(myfile));
		flash_close(myfile);
	}
	else
	{
		HttpSendError(stream, 404, http_req);
	}

	NutHeapFree(data);
}

////////////////////////////////////////////////////////////////////////////////
int ReadLine(u_char * buf, FILE * stream)
{
	char   *cp;

	fgets(buf, PARSE_LINE_BUF, stream);
	if ((cp = strchr(buf, '\r')) != 0)
		*cp = 0;
	if ((cp = strchr(buf, '\n')) != 0)
		*cp = 0;
	return strlen(buf);
}

////////////////////////////////////////////////////////////////////////////////
int ParseRequest(FILE * stream, HttpRequestS * req, long *cont_len)
{
	char   *cp, *line;
	int     method = -1;
	///..........................dodelat posilani erroru
	//line = 0;
	*cont_len = 0;

	if ((line = NutHeapAlloc(PARSE_LINE_BUF)) == 0)
		goto fini;
	if (ReadLine(line, stream) > 0)
	{
		LOG_MSG("..........%s", line);
		if (strncasecmp(line, "GET", 3) == 0)
			method = METHOD_GET;
		else if (strncasecmp(line, "HEAD", 3) == 0)
			method = METHOD_HEAD;
		else if (strncasecmp(line, "POST", 3) == 0)
			method = METHOD_POST;
		else if (strncasecmp(line, "PUT", 3) == 0)
			method = METHOD_PUT;
		else
		{
			method = -1;
			goto fini;
		}

		if ((cp = strchr(line, '?')) != 0)
			*cp = 0;
		if ((cp = strchr(line, '/')) != 0)
		{
			parse_strcpy(req->url, cp + 1);
		}
	}

	while (ReadLine(line, stream) > 0)		//line with zero length terminates header (double CR,LF)
	{
		//LOG_MSG("\nLine:%s", line);
		if (strncasecmp(line, "Content-length: ", 16) == 0)
			*cont_len = atol(line + 16);
		else if (strncasecmp(line, "Authorization:", 14) == 0)
		{
			if (req->auth == 0)
			{
				cp = &line[14];
				while (*cp == ' ' || *cp == '\t')
					cp++;
				if ((req->auth = NutHeapAlloc(strlen(cp) + 1)) == 0)
					break;
				strcpy(req->auth, cp);
				//LOG_MSG("\nAuth:%s", req->auth);
			}
		}
	}
  fini:
	if (line)
		NutHeapFree(line);
	return method;
}

////////////////////////////////////////////////////////////////////////////////
static void ProcessRequest(FILE * stream)
{
	HttpRequestS http_req;
	char   *buf = 0;
	int     method;
	long    total_len;

	if ((http_req.url = NutHeapAlloc(256)) == 0)
		goto fini;
	http_req.auth = NULL;					//memset(&http_req, 0, sizeof(http_req)) (above .url assignment)

	total_len = 0;
	method = ParseRequest(stream, &http_req, &total_len);

	/*
	 * Process file.
	 */

	if ((method == METHOD_PUT) && (gFileSysWriteEnable == 1))
	{

		if ((buf = NutHeapAlloc(RECEIVE_FILE_BUF)) == 0)	//+1+1 --> ">\0"
			goto fini;

		long    n, delka = total_len;
		Tflash_FILE *myfile;

		if ((myfile = flash_open(http_req.url, F_OPENAS_WRITE)) != NULL)
		{
			clearerr(stream);				// do nothing
			do
			{
				n = fread(buf, 1, RECEIVE_FILE_BUF, stream);
				delka -= n;
				if (!ferror(stream))		//ferror - nedela nic stale 0
				{
					if (flash_write(myfile, buf, n) >= 0)
					{
						//                          printf_P(PSTR("\n\rwriting %d"),n);
					}
					else
					{
						//                          printf_P(PSTR("\n\rFlash memory is full"));
						break;
					}
				}
				else
				{
					break;
				}
			}
			while (delka);					//(feof(stream)); // feof nedla nic stale 0

			if (delka == 0)
			{
				HeaderSuccFileTrf(stream);
				flash_close(myfile);
				//                  printf_P(PSTR("\n\rWrite Sucessfully "));
			}
			else
			{
				flash_delete(myfile);		// smaze posledni soubor
				//                  printf_P(PSTR("\n\rWrite error "));
				HttpSendError(stream, 413, &http_req);
			}
		}
		else
			HttpSendError(stream, 500, &http_req);


		if (buf)
			NutHeapFree(buf);

	}


	else if (method == METHOD_GET)
	{
		if ((strlen(http_req.url) == 0))
		{
			strcpy(http_req.url, "index.htm");	//default file name request
		}

#if 1										//DF 22.10.2003 - debugging info
		if (!strcasecmp(http_req.url, "info.htm"))
		{
			SendXMLHeader(stream);
			ShowThreads(stream);
		}
#endif
		else
		{
			SendWebFile_Flash(stream, &http_req);
			//            HttpSendError(stream, 404, &http_req);
			LOG_MSG("\n** ERROR: Page %s not found", http_req.url);
		}
	}

  fini:
	if (http_req.url)
		NutHeapFree(http_req.url);
	if (http_req.auth)
		NutHeapFree(http_req.auth);
	if (buf)
		NutHeapFree(buf);
}

////////////////////////////////////////////////////////////////////////////////
/*
 * The endless loop in this thread waits for a client connect,
 * processes the HTTP request and disconnects. Nut/Net doesn't
 * support a server backlog. If one client has established a
 * connection, further connect attempts will be rejected.
 * Typically browsers open more than one connection in order
 * to load images concurrently. So we run this routine by
 * several threads.
 */
THREAD(webserv, arg)
{
	TCPSOCKET *sock;
	FILE   *stream;
#if defined(LANLOG) || defined(COMMLOG)
	u_char  id = (u_char) ((u_short) arg);
	int     flag = 0;
#endif
	static const u_short tcp_mss = HTTPD_TCP_MSS;

	u_long  ul;
	ul = 3000;


	///original app example httpserv.c does not have priority setting (uses default 64)
	NutThreadSetPriority(HTTP_PRIO /* + id */ );
	/*
	 * TCP socket opened, then 2 others, got opened
	 * and THEN data started to be send on FIRST connection (socket)
	 */

	for (;;)
	{
		if ((sock = NutTcpCreateSocket()) == 0)
		{
			LOG_MSG("%u. Cannot create socket\n", id);
			NutSleep(5000);
			continue;
		}
#ifdef HTTP_DEBUG
		NutThreadSetPriority(HTTP_PRIO_HIGH);
		LOG_MSG("PostOpen  [%u]\n", id);
		ShowSockets();
		NutThreadSetPriority(HTTP_PRIO);
#endif
		if (NutTcpSetSockOpt(sock, TCP_MAXSEG, &tcp_mss, sizeof(tcp_mss)))
			LOG_MSG("%u. Cannot set MSS sock opt\n", id);

		if (NutTcpSetSockOpt(sock, SO_SNDBUF, &tcp_mss, sizeof(tcp_mss)))
			LOG_MSG("%u. Cannot set output buffer sock opt\n", id);

#if defined(LANLOG) || defined(COMMLOG)
		if (!flag)
		{
			LOG_MSG("\n%d ...heap", NutHeapAvailable());
			LOG_MSG("%u. started\n", id);
			flag = 1;
		}
#endif

		NutTcpSetSockOpt(sock, SO_SNDTIMEO, &ul, sizeof(ul));
		NutTcpAccept(sock, gBoardDatap->http_port);	// Listen on http port (default:80). If we return, we got a client.
		//      LOG_MSG("\nClient connect");

		/*
		 * Check if IP address allowed.
		 */
#ifdef HTTP_DEBUG
		NutThreadSetPriority(HTTP_PRIO_HIGH);	//to have all debug outputs from 1 thread together
#endif
		//LOG_MSG("\nFrom IP:%s", inet_ntoa(sock->so_remote_addr));

		//      if (IPAccessCheck(sock->so_remote_addr, gHttpAccess, ALLOWED_IP_RANGE_COUNT))
		{
			//LOG_MSG("\nRequest passed");
			/*
			 * Wait until at least 8 kByte of free RAM is available. This will
			 * keep the client connected in low memory situations.
			 */
			while (NutHeapAvailable() < /*8192 */ 4096)
			{
				LOG_MSG("[%u] Low mem\n", id);
				printf_P(PSTR("Low mem\r\n"));
				NutSleep(1000);
			}

			if ((stream = _fdopen((int)sock, "r+b")) == 0)
			{
				LOG_MSG("[%u] Cannot open stream\n", id);
			}
			else
			{
				LOG_MSG("\n[%u] START" /*" PROCESS REQUEST" */ "\n", id);

				ProcessRequest(stream);
				fflush(stream);				//final flush
				fclose(stream);

				LOG_MSG("\n[%u] END" /*" PROCESS REQUEST" */ "\n", id);
			}
		}

#ifdef HTTP_DEBUG
		LOG_MSG("Pre Close [%u]\n", id);
		ShowSockets();
		NutThreadSetPriority(HTTP_PRIO);
#endif

		NutTcpCloseSocket(sock);

#ifdef HTTP_DEBUG
		NutThreadSetPriority(HTTP_PRIO_HIGH);
		LOG_MSG("PostClose [%u]\n", id);
		ShowSockets();
		NutThreadSetPriority(HTTP_PRIO);
#endif
	}
}

void HttpInit(void)
{
	u_char  i;
	/*
	 * Start HTTP_THREAD_COUNT server threads.
	 */
	for (i = 0; i < HTTP_THREAD_COUNT; i++)
	{
		char   *thname = "httpd0";

		thname[5] = '0' + i;
		LOG_MSG("Starting %u. web server ...\n", i);


		if (NutThreadCreate
			(thname, webserv, (void *)(u_short) i, 1024 /*<-for sax parser */ ) == NULL)
		{
			LOG_MSG("\n\r Low mem for %s", thname);
			return;
		}
	}

#ifdef HTTP_DEBUG
	LOG_MSG("AfterCreate\n");
	ShowSockets();
#endif
}
