/*
 * Copyright (C) 2003-4 by Dusan Ferbas - DF Soft
 * <a href="www.dfsoft.cz">www.dfsoft.cz</a>
 *
 * Permission to use, copy, modify, and distribute this software
 * for any purpose and without fee is hereby granted, provided
 * that the above copyright notices appear in all copies and that
 * both the copyright notice and this permission notice appear in
 * supporting documentation and that the name of DF Soft may not be
 * used to endorse or promote products derived from this
 * software without specific prior written permission.
 *
 * This software is provided "as is" without express or implied
 * warranty. We are not liable for anything caused by using this software.
 *
 * author <a href="mailto:dferbas@dfsoft.cz">Dusan Ferbas</a>
 */

/****************************************************************************
 *
 * File:         setup.c
 * Author(s):    Dusan Ferbas
 * Project:      Charon SNMP Implementation
 * Description:  RS232 setup
 *
 ****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <ctype.h>
#include <sys/timer.h>
#include <net/route.h>

#include "setup.h"
#include "../utilities.h"
#include "../eeparams/eeparams.h"
#include "../spi_flash/file_sys.h"


#define MAC_FULL_EDIT						//enable full 6 byte edit



/*
 * Normally \n is expanded into \r\n by Nut OS.
 * But we use uart0 for communication with CCM and we need exact \r \n control.
 */

static prog_char info_P[] =
	"\r\nEEPROM Parameters Demo " VER_NUM_STR " (C) 2004 by www.dfsoft.cz\r\n"
	"\r\n"
	"Application demonstrates simple EEPROM usage for storing parameters.\r\n"
	"You can change or reorder EEPROM structure and parameter values remain preserved.\n"
	"\r\n" "Maximum parameters survive between modified data structure versions.\n";

static prog_char title_P[] =
	"\r\n    --- commands ---\r\n\n"
	"  h,?.. help\r\n"
	"  ------------------------------\r\n"
	"  i ... enter Nut IP address\r\n"
	"  k ... enter Nut netmask\r\n"
	"  g ... enter Nut gateway address\r\n"
	"  ------------------------------\r\n"
	"  m ... MAC edit\r\n"
	"  ------------------------------\r\n"
	"  c ... clear file system\r\n"
	"  f ... file system write enable/disable\r\n"
	"  l ... list file system table\r\n"
	"  ------------------------------\r\n"
	"  d ... load default setup\r\n"
	"  e ... EEPROM erase\r\n"
	"  r ... system reboot\r\n"
	"  x ... exit\r\n";


void    GetIP(const prog_char * info_str, IP_ADDR * ip_addrp);
static void GetMACAddr(void);

void GetIPAddr(void)
{
	GetIP(PSTR("IP address"), &my_ip_addr);

	eeprom_put(eeprom_idx + (u_short) CFG_ITEM_OFFS(aux_my_ip_addr)	/*EE_IP_ADDR_OFFSET */
			   , CFG_ITEM_ADDR(aux_my_ip_addr), CFG_ITEM_LEN(aux_my_ip_addr));
	//(void)set_myIpAddr((u_char *) & my_ip_addr, TRUE);

	printf_P(PSTR("\r\nOK .. IP is %u.%u.%u.%u"), my_ip_addr.ip_addr[0],
			 my_ip_addr.ip_addr[1], my_ip_addr.ip_addr[2], my_ip_addr.ip_addr[3]);

}

void GetIPMask(void)
{
	GetIP(PSTR("IP mask"), &my_ip_mask);

	eeprom_put(eeprom_idx + (u_short) CFG_ITEM_OFFS(aux_my_ip_mask),
			   CFG_ITEM_ADDR(aux_my_ip_mask), CFG_ITEM_LEN(aux_my_ip_mask));
	//(void)set_myIpMask((u_char *) & netmask, TRUE);

	printf_P(PSTR("\r\nOK .. mask is %u.%u.%u.%u"), my_ip_mask.ip_addr[0],
			 my_ip_mask.ip_addr[1], my_ip_mask.ip_addr[2], my_ip_mask.ip_addr[3]);

}

void GetIPGw(void)
{
	GetIP(PSTR("gateway address"), &gw_ip_addr);

	eeprom_put(eeprom_idx + (u_short) CFG_ITEM_OFFS(aux_gw_ip_addr),
			   CFG_ITEM_ADDR(aux_gw_ip_addr), sizeof(IP_ADDR));
	//(void)set_myIpGway((u_char *) & gw_ip_addr, TRUE);

	printf_P(PSTR("\r\nOK .. Gateway is %u.%u.%u.%u"), gw_ip_addr.ip_addr[0],
			 gw_ip_addr.ip_addr[1], gw_ip_addr.ip_addr[2], gw_ip_addr.ip_addr[3]);
}

static void ShowInfo(void)
{
	printf_P(info_P);
	printf_P(title_P);
	//fflush(uart);
}


void Setup(FILE * uart)
{
	char    c;

	stdout = stdin = uart;

	ShowInfo();

	while (1)
	{
		printf_P(PSTR("\r\nNut>"));			// puts_P() adds \n after output
		c = getchar();

		switch (c)
		{
		case 'i':
			GetIPAddr();
			break;
		case 'k':
			GetIPMask();
			break;
		case 'g':
			GetIPGw();
			break;
		case 'm':
			(void)GetMACAddr();
			NutNetIfConfig("eth0", my_eth_addr.eth_addr /*mac */ ,
						   *(u_long *) & my_ip_addr, *(u_long *) & my_ip_mask);
			break;

		case 'c':
			printf_P(PSTR("\r\nFile table erasing..."));
			flash_clear();
			break;
		case 'f':
			gFileSysWriteEnable = 1 - gFileSysWriteEnable;
			printf_P(PSTR("\r\nFile system write %s"),
					 gFileSysWriteEnable ? "Enabled" : "Disabled");
			break;
		case 'l':
			display_table(ACTUALL_TABLE);
			MemStat();
			break;
		case '\r':
		case 'h':
		case '?':
			ShowInfo();
			break;

		case 'd':
			printf_P(PSTR("Setting, please wait ..."));
			(void)eeprom_load(1);

			if (!eeprom_verify())
			{
				printf_P(PSTR("\r\n EEPROM failure!"));
			}
			break;
		case 'e':
			printf_P(PSTR("\r\nEEPROM erasing..."));
			eeprom_clear();
			unit_reboot();
			break;

		case 'x':
			printf_P(PSTR("Exiting, starting application ..."));
			return;
			//break;
		case 'r':
			printf_P(PSTR("Restarting ..."));
			unit_reboot();
			break;

		default:
			break;
		}
	}
}

void GetIP(const prog_char * info_str, IP_ADDR * ip_addrp)
{
	u_char  dummy[10];						//this will hold number (1 byte of IP address) read in
	u_char  c, i, j, k;
	u_char  filled = 0;

	printf_P(PSTR("Enter %P (%u.%u.%u.%u):"), info_str, ip_addrp->ip_addr[0],
			 ip_addrp->ip_addr[1], ip_addrp->ip_addr[2], ip_addrp->ip_addr[3]);

	for (j = 0; j < 4; j++)
	{
		for (i = 0; i < 3; i++)
		{
			//printf("\r\ni=%u,j=%u,f=%u,s='%s'", (int)i, (int)j, (int)filled, dummy);

			c = getchar();

			// backspace support
			if (c == '\b')
			{
				if (filled)
				{
					i = strlen(dummy);
					filled = 0;				//<BS> clears this flag
				}

				if (i > 0)
				{
					i--;
					printf_P(PSTR("\b \b"));
				}
				else
				{
					if (j > 0)
					{
						j--;
						printf_P(PSTR("\b\b  \b\b"));
						sprintf_P(dummy, PSTR("%u"), ip_addrp->ip_addr[j]);
						i = strlen(dummy) - 1;
						//fill buffer for edit
					}
				}

				dummy[i] = (u_char) '\0';	//dismiss chr in dummy buffer in case <Enter> will follow

				i--;						//will be incremented in a loop
				//continue;
			}
			else
			{
				if (c == '.')
				{
					if (i > 0)				//ignore
						//will be processed below
						break;

					//dismiss character
					i--;
				}
				else if (c == '\r')			//<CR>
				{
					if (j < 3)
					{
						if (!i && !filled)	//special case - 1st input in a field and it is <CR>
						{
							sprintf_P(dummy, PSTR("%u"), ip_addrp->ip_addr[j]);
							//fill buffer for atoi() (below)
							printf(dummy);	//display on screen (puts adds <CR><LF>)
							filled = 1;
							continue;
						}
						filled = 1;
					}

					break;
				}
				else if (c == '\n')			//ignore <LF>
					i--;
				else
				{
					if (filled)
					{
						if ((k = strlen(dummy)) > 2)
						{
							putchar('\7');	//bell
							i--;
							continue;
						}
						else
						{
							filled = 0;
							i = k;
						}
					}

					dummy[i] = c;
					putchar(c);				//echo input on screen
					dummy[i + 1] = (u_char) '\0';
				}
			}
		}

		if (j < 3)							//input exceeded number width or dot or Enter typed, put delimiter
			putchar('.');

		ip_addrp->ip_addr[j] = atoi(dummy);

		if (filled && j < 3)
		{
			sprintf_P(dummy, PSTR("%u"), ip_addrp->ip_addr[j + 1]);
			//fill buffer for edit
			printf(dummy);					//display on screen (puts adds \n)
		}
		else
			*dummy = '\0';					//in case if only <Enter> follows
	}
}


#ifdef MAC_FULL_EDIT
#define	MAC_START_POS	0
#else
/* no need to seed 1st 3 bytes, they are seeded with eeprom.c logic */
#define	MAC_START_POS	3
#endif


static u_char atox(u_char c)
{
	c = toupper(c) - '0';

	if (c > 9)
		c -= 'A' - '0' - 10;

	return c;
}

static void GetMACAddr(void)
{
	char    c, k;
	int     i, j;
	u_char  filled = 0;
	char    dummy[10];
	//  static u_char atox(u_char c);

	printf("Enter Web51 MAC address ("
#ifdef MAC_FULL_EDIT
		   "%.2x:%.2x:%.2x:"
#endif
		   "%.2x:%.2x:%.2x):"
#ifdef MAC_FULL_EDIT
		   , my_eth_addr.eth_addr[0], my_eth_addr.eth_addr[1], my_eth_addr.eth_addr[2]
#endif
		   , my_eth_addr.eth_addr[3], my_eth_addr.eth_addr[4], my_eth_addr.eth_addr[5]);

	for (j = MAC_START_POS; j < 6; j++)
	{
		for (i = 0; i < 2; i++)
		{
			//printf("\ni=%u,j=%u,f=%u,s='%s'", (int)i, (int)j, (int)filled, dummy);

			c = getchar();


			/* backspace support */
			if (c == '\b')
			{
				if (filled)
				{
					i = strlen(dummy);
					filled = 0;				//<BS> clears this flag
				}

				if (i > 0)
				{
					i--;
					printf("\b \b");
				}
				else
				{
					if (j > MAC_START_POS)
					{
						j--;
						printf("\b\b  \b\b");
						sprintf(dummy, "%.2x", my_eth_addr.eth_addr[j]);
						i = strlen(dummy) - 1;
						//fill buffer for edit
					}
				}

				dummy[i] = (u_char) '\0';	//dismiss chr in dummy buffer in case <Enter> will follow

				i--;						//will be incremented in a loop
				//continue;
			}
			else
			{
				if (c == ':')
				{
					if (i > 0)				//ignore
						//will be processed below
						break;

					//dismiss character
					i--;
				}
				else if (c == '\r')			//<CR>
				{
					if (j < 5)
					{
						if (!i && !filled)	//special case - 1st input in a field and it is <CR>
						{
							sprintf(dummy, "%.2x", my_eth_addr.eth_addr[j]);
							//fill buffer for atoi() (below)
							printf(dummy);	//display on screen (puts adds <CR><LF>)
							filled = 1;
							continue;
						}
						filled = 1;
					}

					break;
				}
				else if (c == '\n')			//ignore <LF>
					i--;
				else
				{
					if (filled)
					{
						if ((k = strlen(dummy)) > 1)
						{
							putchar('\7');	//bell
							i--;
							continue;
						}
						else
						{
							filled = 0;
							i = k;
						}
					}

					dummy[i] = c;
					putchar(c);				//echo input on screen
					dummy[i + 1] = (u_char) '\0';
				}
			}
		}

		if (j < 5)							//input exceeded number width or dot or Enter typed, put delimiter
			putchar(':');

		my_eth_addr.eth_addr[j] = (atox(dummy[0]) << 4) + atox(dummy[1]);

		if (filled && j < 5)
		{
			sprintf(dummy, "%.2x", my_eth_addr.eth_addr[j + 1]);
			//fill buffer for edit
			printf(dummy);					//display on screen (puts adds <CR><LF>)
		}
		else
			*dummy = '\0';					//in case if only <Enter> follows
	}

	//save into EEPROM
	eeprom_put(EE_TABLEI +
			   //    (u_int)((u_char *)&my_eth_addr - (u_char *)&ee_tbl_def)/*EE_ETH_ADDR_OFFSET*/
			   (u_int) offsetof(ee_tbl_data, e_tbl.dta.enet_addr)	/*EE_ETH_ADDR_OFFSET */
			   , (u_char *) & my_eth_addr, sizeof(ETH_ADDR));
}
